
# q-expansion of newform 765.2.t.e, downloaded from the LMFDB on 16 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 765
weight = 2
poly_data = [100, 0, 464, 0, 776, 0, 572, 0, 188, 0, 24, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-920, -1052, -2692, -3296, -2770, -3016, -1136, -1078, -160, -142, -7, -6], 40], [[1080, 0, 3438, 0, 3440, 0, 1344, 0, 185, 0, 8, 0], 20], [[-1320, 0, -4204, 0, -4110, 0, -1552, 0, -210, 0, -9, 0], 20], [[0, -1666, 0, -5096, 0, -4948, 0, -1883, 0, -256, 0, -11], 20], [[0, -1646, 0, -5096, 0, -4948, 0, -1883, 0, -256, 0, -11], 20], [[3560, -1052, 11100, -3296, 10990, -3016, 4240, -1078, 580, -142, 25, -6], 40], [[4420, -1284, 13236, -4456, 12380, -4642, 4558, -1838, 610, -254, 26, -11], 40], [[4420, 1284, 13236, 4456, 12380, 4642, 4558, 1838, 610, 254, 26, 11], 40], [[-5560, 2372, -17056, 7500, -16460, 7126, -6208, 2630, -840, 352, -36, 15], 40], [[5560, 2372, 17056, 7500, 16460, 7126, 6208, 2630, 840, 352, 36, 15], 40], [[-3560, 5596, -11100, 16804, -10990, 15928, -4240, 5922, -580, 796, -25, 34], 40]]

hecke_ring_character_values = [[596, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [307, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [496, [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0], [-1, 0, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0], [0, -1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 1], [0, 0, 0, -2, 0, 1, 0, -1, 0, 0, 0, 0], [0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0], [0, -1, 1, -2, 0, -1, 0, 0, -3, 0, 0, 1], [-1, -1, 0, 0, -1, 0, 0, 0, 0, 0, -2, -1], [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, -1, 1], [0, 1, -1, 0, 0, -1, 0, 0, 0, 0, 0, -1], [-2, 1, 0, 0, 2, 0, -2, 0, 0, 2, 0, -1], [0, 0, 2, -1, 0, 0, 0, -2, -2, 0, 0, 0], [0, -1, 0, -1, 0, 2, 0, 1, -1, 0, 0, 1], [0, 0, 4, -2, 0, 0, 0, -1, -1, 0, 0, 0], [0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0], [-4, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0], [0, -1, 0, -1, 0, 4, 0, 1, -1, 0, 0, 1], [3, 1, 0, 0, 3, 0, 0, 0, 0, 0, -1, 1], [0, -2, -2, 0, 0, -2, 0, 4, 0, 0, 0, 2], [0, -2, 0, 0, 0, 0, 4, 0, 0, -1, 0, 2], [0, 0, 2, 1, 0, 0, 0, -1, -1, 0, 0, 0], [0, -5, 0, 0, 0, 0, 5, 0, 0, 2, -2, 0], [0, 5, 1, 0, 0, 1, 0, -4, 0, 0, 0, -5], [2, 1, 0, 0, 0, 0, -1, 0, 0, -2, 2, 0], [0, -5, 0, -5, 0, 0, 0, 4, -4, 0, 0, 5], [0, 0, 3, 0, 0, 3, 0, 3, 0, 0, 0, 0], [-6, -1, 0, 0, 6, 0, 2, 0, 0, 2, 0, 1], [0, 0, -2, 0, 0, 2, 0, 0, -2, 0, 0, 0], [0, 0, 2, 3, 0, 0, 0, 0, 0, 0, 0, 0], [0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 5, 4], [0, -3, 0, -3, 0, 0, 0, -1, 1, 0, 0, 3], [5, -3, 0, 0, 5, 0, 0, 0, 0, 0, -1, -3], [-10, 0, 0, 0, 0, 0, 0, 0, 0, -1, 1, 0], [0, 0, 0, 0, -8, 0, -2, 0, 0, -2, -2, -2], [0, -2, 0, -2, 0, -6, 0, 3, -3, 0, 0, 2], [0, -4, 0, -8, 0, 0, 0, 0, -1, 0, 0, 4], [0, 2, 0, 0, 0, 0, 0, -3, 0, 0, 0, -2], [0, 1, -5, 0, 0, -5, 0, 8, 0, 0, 0, -1], [0, 0, 0, 0, 4, 0, -8, 0, 0, -3, -3, -8], [5, 1, 0, 0, -5, 0, -2, 0, 0, -4, 0, -1], [8, 0, 0, 0, 0, 0, 0, 0, 0, 1, -1, 0], [0, 3, 2, 6, 0, -2, 0, 0, 4, 0, 0, -3], [0, -2, 1, -4, 0, -1, 0, 0, 2, 0, 0, 2], [-2, 4, 0, 0, -2, 0, 0, 0, 0, 0, 1, 4], [-8, -1, 0, 0, 8, 0, 2, 0, 0, 3, 0, 1], [0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0], [0, 0, 4, 0, 0, -4, 0, 0, -7, 0, 0, 0], [0, 0, 0, 0, -12, 0, 1, 0, 0, 0, 0, 1], [0, -4, 5, 0, 0, 5, 0, -2, 0, 0, 0, 4], [16, 4, 0, 0, 0, 0, -4, 0, 0, -2, 2, 0], [6, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0], [-2, 0, 0, 0, 0, 0, 0, 0, 0, -3, 3, 0], [0, 0, -10, 9, 0, 0, 0, 5, 5, 0, 0, 0], [0, 0, 2, 1, 0, 0, 0, 4, 4, 0, 0, 0], [0, 3, 0, 0, 0, 0, 0, 0, 0, 0, -4, 3], [-6, 8, 0, 0, 0, 0, -8, 0, 0, -2, 2, 0], [0, 0, 8, 0, 0, 8, 0, -2, 0, 0, 0, 0], [0, 0, 0, 0, -10, 0, 0, 0, 0, -3, -3, 0], [0, -5, 8, -10, 0, -8, 0, 0, -7, 0, 0, 5], [0, 8, 0, 8, 0, -2, 0, -6, 6, 0, 0, -8], [0, -5, 0, -5, 0, -10, 0, 2, -2, 0, 0, 5], [11, 3, 0, 0, 11, 0, 0, 0, 0, 0, 3, 3], [0, 0, -4, 0, 0, 4, 0, 0, -4, 0, 0, 0], [0, -3, -2, -6, 0, 2, 0, 0, 0, 0, 0, 3], [0, 0, 0, 0, -4, 0, -6, 0, 0, 0, 0, -6], [0, 5, -5, 0, 0, -5, 0, 0, 0, 0, 0, -5], [0, -1, 4, -2, 0, -4, 0, 0, -1, 0, 0, 1], [0, 0, 0, 0, 6, 0, -2, 0, 0, -1, -1, -2], [0, 4, 0, 4, 0, -4, 0, 6, -6, 0, 0, -4], [0, 0, 0, 0, 4, 0, 2, 0, 0, 5, 5, 2], [0, -6, -2, -12, 0, 2, 0, 0, -5, 0, 0, 6], [0, 7, 0, 7, 0, 2, 0, 1, -1, 0, 0, -7], [-2, -1, 0, 0, -2, 0, 0, 0, 0, 0, -1, -1], [0, 0, -6, -5, 0, 0, 0, -7, -7, 0, 0, 0], [0, 0, 0, 0, -14, 0, -5, 0, 0, 3, 3, -5], [0, -3, -6, -6, 0, 6, 0, 0, -2, 0, 0, 3], [11, 3, 0, 0, -11, 0, -6, 0, 0, -4, 0, -3], [-2, 2, 0, 0, 0, 0, -2, 0, 0, -2, 2, 0], [-6, -7, 0, 0, 6, 0, 14, 0, 0, -3, 0, 7], [10, 7, 0, 0, 0, 0, -7, 0, 0, -1, 1, 0], [4, 0, 0, 0, -4, 0, 0, 0, 0, -3, 0, 0], [0, 0, 4, 3, 0, 0, 0, -3, -3, 0, 0, 0], [3, 4, 0, 0, 3, 0, 0, 0, 0, 0, 1, 4], [0, -9, 0, -9, 0, -6, 0, 6, -6, 0, 0, 9], [6, -4, 0, 0, -6, 0, 8, 0, 0, -4, 0, 4], [0, 0, -12, -3, 0, 0, 0, 11, 11, 0, 0, 0], [0, 0, 0, 0, 2, 0, -2, 0, 0, 7, 7, -2], [0, 3, 0, 3, 0, -4, 0, -4, 4, 0, 0, -3], [0, 0, 0, -5, 0, 0, 0, -8, -8, 0, 0, 0], [-12, -5, 0, 0, -12, 0, 0, 0, 0, 0, -1, -5], [0, -8, -6, -16, 0, 6, 0, 0, 3, 0, 0, 8], [0, 0, 0, 0, 26, 0, 4, 0, 0, 1, 1, 4], [2, 1, 0, 0, -2, 0, -2, 0, 0, -3, 0, -1], [0, 5, -4, 10, 0, 4, 0, 0, 1, 0, 0, -5], [0, 10, 0, 0, 0, 0, -10, 0, 0, 2, -2, 0], [1, 3, 0, 0, -1, 0, -6, 0, 0, 6, 0, -3], [0, 11, 0, 11, 0, 4, 0, -1, 1, 0, 0, -11], [-9, 0, 0, 0, -9, 0, 0, 0, 0, 0, 0, 0], [0, -1, -1, 0, 0, -1, 0, -7, 0, 0, 0, 1], [0, 11, 0, 11, 0, 0, 0, -9, 9, 0, 0, -11], [0, 0, 12, 7, 0, 0, 0, 1, 1, 0, 0, 0], [0, 0, 0, 0, -6, 0, 2, 0, 0, 0, 0, 2], [-2, -2, 0, 0, 2, 0, 4, 0, 0, 5, 0, 2], [0, -9, 0, -9, 0, 10, 0, -5, 5, 0, 0, 9], [0, 0, 4, -11, 0, 0, 0, -3, -3, 0, 0, 0], [0, 0, -12, 6, 0, 0, 0, 7, 7, 0, 0, 0], [-12, -8, 0, 0, 0, 0, 8, 0, 0, 1, -1, 0], [0, -4, 0, 0, 0, 0, 8, 0, 0, -4, 0, 4], [0, -1, -9, 0, 0, -9, 0, 5, 0, 0, 0, 1], [0, 10, 0, 10, 0, 10, 0, -6, 6, 0, 0, -10], [0, -4, -2, 0, 0, -2, 0, 4, 0, 0, 0, 4], [11, -3, 0, 0, -11, 0, 6, 0, 0, -9, 0, 3], [0, 0, 0, 0, -4, 0, -6, 0, 0, 6, 6, -6], [-28, 2, 0, 0, -28, 0, 0, 0, 0, 0, 2, 2], [0, 4, -3, 0, 0, -3, 0, -1, 0, 0, 0, -4], [0, 5, 0, 5, 0, -6, 0, 5, -5, 0, 0, -5], [0, -3, -9, -6, 0, 9, 0, 0, -6, 0, 0, 3], [22, -2, 0, 0, 0, 0, 2, 0, 0, -6, 6, 0], [0, 0, 0, 0, -10, 0, 10, 0, 0, 7, 7, 10], [0, 12, -6, 24, 0, 6, 0, 0, 12, 0, 0, -12], [0, 4, 0, 0, 0, 0, 0, 4, 0, 0, 0, -4], [0, -3, 3, 0, 0, 3, 0, -13, 0, 0, 0, 3], [-11, 2, 0, 0, 11, 0, -4, 0, 0, 1, 0, -2], [12, -9, 0, 0, 0, 0, 9, 0, 0, 5, -5, 0], [-13, 0, 0, 0, -13, 0, 0, 0, 0, 0, -8, 0], [-11, 4, 0, 0, -11, 0, 0, 0, 0, 0, -5, 4], [0, 5, 0, 5, 0, -2, 0, 6, -6, 0, 0, -5], [0, 0, -2, 18, 0, 0, 0, 4, 4, 0, 0, 0], [0, 0, 0, 0, -6, 0, 0, 0, 0, 0, 0, 0], [0, 13, -1, 0, 0, -1, 0, -1, 0, 0, 0, -13], [-5, 1, 0, 0, -5, 0, 0, 0, 0, 0, -11, 1], [0, 0, -18, 5, 0, 0, 0, 9, 9, 0, 0, 0], [-6, -1, 0, 0, 0, 0, 1, 0, 0, 8, -8, 0], [-16, -9, 0, 0, 0, 0, 9, 0, 0, 6, -6, 0], [0, 0, -14, 1, 0, 0, 0, 11, 11, 0, 0, 0], [0, -3, 10, 0, 0, 10, 0, 7, 0, 0, 0, 3], [0, 0, -24, 0, 0, 0, 0, 10, 10, 0, 0, 0], [-7, -6, 0, 0, 7, 0, 12, 0, 0, -2, 0, 6], [-5, 1, 0, 0, -5, 0, 0, 0, 0, 0, -1, 1], [8, -4, 0, 0, 8, 0, 0, 0, 0, 0, 6, -4], [0, 6, -11, 12, 0, 11, 0, 0, 13, 0, 0, -6], [0, -2, -1, -4, 0, 1, 0, 0, -15, 0, 0, 2], [2, -2, 0, 0, 0, 0, 2, 0, 0, 6, -6, 0], [-15, -3, 0, 0, 15, 0, 6, 0, 0, 7, 0, 3], [0, 7, 15, 0, 0, 15, 0, -10, 0, 0, 0, -7], [0, 0, -8, 0, 0, 8, 0, 0, 6, 0, 0, 0], [0, 0, 0, 0, 22, 0, 2, 0, 0, 9, 9, 2], [0, 5, 0, 5, 0, 16, 0, -1, 1, 0, 0, -5], [0, -10, 0, -20, 0, 0, 0, 0, -14, 0, 0, 10], [4, 0, 0, 0, 4, 0, 0, 0, 0, 0, 6, 0], [0, -3, 0, -3, 0, 22, 0, -4, 4, 0, 0, 3], [0, -12, -7, 0, 0, -7, 0, 11, 0, 0, 0, 12], [0, -6, 14, -12, 0, -14, 0, 0, -9, 0, 0, 6], [-19, -2, 0, 0, 19, 0, 4, 0, 0, 9, 0, 2], [12, -12, 0, 0, 0, 0, 12, 0, 0, 7, -7, 0], [-2, -1, 0, 0, 2, 0, 2, 0, 0, -6, 0, 1], [0, 0, 4, 2, 0, 0, 0, -11, -11, 0, 0, 0], [9, 0, 0, 0, -9, 0, 0, 0, 0, -14, 0, 0], [0, 0, 8, 0, 0, 8, 0, 3, 0, 0, 0, 0], [0, 13, 0, 13, 0, 4, 0, -13, 13, 0, 0, -13], [0, 0, -2, -3, 0, 0, 0, -4, -4, 0, 0, 0], [0, 0, 0, 0, 12, 0, 8, 0, 0, -1, -1, 8], [0, 0, 2, 0, 0, 0, 0, -7, -7, 0, 0, 0], [0, -1, -4, 0, 0, -4, 0, 7, 0, 0, 0, 1], [19, 2, 0, 0, 19, 0, 0, 0, 0, 0, -7, 2], [0, 5, -3, 10, 0, 3, 0, 0, 8, 0, 0, -5]]
